#ifndef FRACTIONH
#define FRACTIONH 

/* Fraction.h -- declarations for fractions

Author:
	K. E. Gorlen
	Computer Systems Laboratory, DCRT
	National Institutes of Health
	Bethesda, MD 20892

Modification History:

*/

#include "Object.h"

extern Class class_Fraction;

class Fraction: public Object {
	int n,d;
	Fraction(int num, int den, int dum) {
		n = (dum,num); d = den;
	}
	void parseFraction(istream&);
	void reduce();
public:
	Fraction(int num =0, int den =1);
	Fraction(double);
	Fraction(istream&);
	Fraction(fileDescTy&,Fraction&);
	Fraction(istream&,Fraction&);
	operator double()		{ return (double)n/d; }
	int denominator()		{ return d; }
	int numerator()			{ return n; }
	
	friend Fraction	operator+(Fraction, Fraction);
	friend Fraction	operator-(Fraction u)			{ return Fraction(-u.n,u.d); }
	friend Fraction operator-(Fraction, Fraction);
	friend Fraction operator*(Fraction, Fraction);
	friend Fraction operator/(Fraction, Fraction);
	friend bool	operator<(Fraction u, Fraction v);
	friend bool	operator>(Fraction u, Fraction v)	{ return v<u; }
	friend bool	operator<=(Fraction u, Fraction v);
	friend bool	operator>=(Fraction u, Fraction v)	{ return v<=u; }
	friend bool	operator==(Fraction u, Fraction v)	{ return u.n == v.n && u.d == v.d; }
	friend bool	operator!=(Fraction u, Fraction v)	{ return !(u==v); }
	
	void operator+=(Fraction u)	{ *this = *this + u; }
	void operator-=(Fraction u)	{ *this = *this - u; }
	void operator*=(Fraction u)	{ *this = *this * u; }
	void operator/=(Fraction u)	{ *this = *this / u; }
	
	bool between(const Fraction& min, const Fraction& max);
	Fraction max(const Fraction&);
	Fraction min(const Fraction&);
	
	virtual int compare(const Object&);
	virtual obid	copy();			// { return shallowCopy(); }
	virtual void	deepenShallowCopy();	// {}
	virtual UNSIGNED hash();
	virtual const Class* isA();
	virtual bool isEqual(const Object&);
	virtual void printOn(ostream& strm);
	virtual void scanFrom(istream& strm);
	virtual const Class* species();
	virtual void storer(fileDescTy&);
	virtual void storer(ostream&);
};

#endif
